if (select(2, UnitClass("player"))) ~= "ROGUE" then return end

local mod = Envenomous:NewModule("Deadly", nil, "AceEvent-3.0", "AceConsole-3.0")
local Media = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Envenomous")
local self = mod
local db

local defaults = {
	profile = {
		deadly = {
			y = 350,
			
			width   = 250,
			height  = 16,
			alpha   = 1,
			scale   = 100.0,
			texture = "BantoBar",
			icon = true,
			iconPosition = 1,
			
			border    = "RothSquare",
			backColor = { 0, 0, 0, 1 },
			barColor = { 34/255, 250/255, 42/255, 0.8 },
			
			textShow     = true,
			textPosition = 2,
			textColor    = { 1, 1, 1, 1 },
			textFont     = "Emblem",
			textSize     = 14,
			
		}
	}
}

-- Frames
local locked = true
local dpBar, dpTimeText, dpParent

-- Localized functions
local GetTime = _G.GetTime
local UnitGUID = _G.UnitGUID
local playerName = nil

-- Settings/infos
local resetValues = true
local spellInfo = GetSpellInfo(57970) -- Deadly Poison IX
local tracking = {}

local function OnUpdate()
	if self.running then
		local currentTime = GetTime()
		local startTime = self.startTime
		local endTime = self.endTime
	
		local remainingTime = endTime - currentTime
		remainingTime = ((remainingTime > 0) and remainingTime or 0) -- Hackity hack hack
	
		if remainingTime == 0 then
			self.running = false
		else
			local perc = remainingTime / 3
			dpBar:SetValue(perc)
			dpTimeText:SetText(("%.1f"):format(remainingTime))
		end
	else
		if dpParent:IsVisible() then
			self:CheckVisibility(true)
		end
	end
end
mod.OnUpdate = OnUpdate
local function OnShow()
	dpParent:SetScript("OnUpdate", OnUpdate)
end
local function OnHide()
	dpParent:SetScript("OnUpdate", nil)
end

function mod:OnInitialize()
	db = LibStub("AceDB-3.0"):New("EnvenomousDB", nil, "Default")
	self.db = db
	
	self.db:RegisterDefaults(defaults)
	
	self:SetEnabledState(false)
end

function mod:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	
	playerName = UnitName("player")
	
	self.locked = locked
	self.startTime = 0
	self.endTime = 0
	self.running = false
	
	if not dpParent then
		dpParent = CreateFrame("Frame", nil, UIParent)
		dpParent:SetFrameStrata("LOW")
		dpParent:SetScript("OnShow", OnShow)
		dpParent:SetScript("OnHide", OnHide)
		dpParent:SetMovable(true)
		dpParent:RegisterForDrag("LeftButton")
		dpParent:SetClampedToScreen(true)
		
		dpBar = CreateFrame("StatusBar", nil, dpParent)
		dpBar:SetFrameStrata("MEDIUM")
		dpTimeText = dpBar:CreateFontString(nil, "OVERLAY")
		
		dpParent:Hide()
	end
	self:ApplySettings()
end

function mod:OnDisable()
	self.running = false
	self.locked = true
	dpParent:Hide()
	dpParent = nil
	
	self:UnregisterAllEvents()
end

-- ---------------------
-- Frame methods
-- ---------------------

function mod:ApplySettings()
	if not self:IsEnabled() then return end
	
	if dpParent then
		local db = db.profile.deadly
		local back = {}
		
		-- dpParent, to which all our stuff is anchored
		dpParent:ClearAllPoints()
		if not db.x then
			db.x = (UIParent:GetWidth() / 2 - (db.width * (db.scale/100.0)) / 2) / (db.scale/100.0)
		end
		dpParent:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", db.x, db.y)
		
		if db.border == "None" then
			dpParent:SetWidth(db.width)
			dpParent:SetHeight(db.height)
			
			back.bgFile = "Interface\\Tooltips\\UI-Tooltip-Background"
			back.tile = true
			back.tileSize = 16
			back.insets = { top = 0, right = 0, bottom = 0, left = 0 }
		else
			dpParent:SetWidth(db.width + 9)
			dpParent:SetHeight(db.height + 10)
			
			back.bgFile = "Interface\\Tooltips\\UI-Tooltip-Background"
			back.tile = true
			back.tileSize = 16
			back.edgeFile = Media:Fetch("border", db.border)
			back.edgeSize = 16
			back.insets = { top = 4, right = 4, bottom = 4, left = 4 }
		end
		dpParent:SetBackdrop(back)
		dpParent:SetBackdropColor(unpack(db.backColor))
		
		dpParent:SetAlpha(db.alpha)
		dpParent:SetScale(db.scale / 100.0)
		
		-- make us an icon!
		dpParent.icon = dpParent.icon or dpParent:CreateTexture(nil, "OVERLAY")
		dpParent.icon:SetPoint("RIGHT", dpParent, "LEFT", 0, 0)
		dpParent.icon:SetHeight(db.height + 5) --scaled 1.1
		dpParent.icon:SetWidth(db.height + 5)
		dpParent.icon:SetTexture([[Interface\ICONS\Ability_Rogue_DualWeild]])
		if db.icon then 
			dpParent.icon:Show() 
		else
			dpParent.icon:Hide()
		end
		
		-- dpBar, the actual timer bar
		dpBar:ClearAllPoints()
		dpBar:SetPoint("CENTER", dpParent, "CENTER")
		dpBar:SetWidth(db.width)
		dpBar:SetHeight(db.height)
		dpBar:SetStatusBarTexture(Media:Fetch("statusbar", db.texture))
		dpBar:SetMinMaxValues(0, 1)
		dpBar:SetStatusBarColor(unpack(db.barColor))
		dpBar:Show()
		
		-- dpTimeText, countdown timer text
		dpTimeText:ClearAllPoints()
		if db.textPosition == 1 then -- LEFT
			dpTimeText:SetPoint("LEFT", dpParent, "LEFT", 5, 0)
			dpTimeText:SetJustifyH("LEFT")
		elseif db.textPosition == 2 then -- CENTER
			dpTimeText:SetPoint("CENTER", dpParent, "CENTER")
			dpTimeText:SetJustifyH("CENTER")
		elseif db.textPosition == 3 then -- RIGHT
			dpTimeText:SetPoint("RIGHT", dpParent, "RIGHT", -5, 0)
			dpTimeText:SetJustifyH("RIGHT")
		end
		dpTimeText:SetFont(Media:Fetch("font", db.textFont), db.textSize)
		dpTimeText:SetTextColor(unpack(db.textColor))
		dpTimeText:SetShadowColor(0, 0, 0, 1)
		dpTimeText:SetShadowOffset(0.8, -0.8)
		dpTimeText:SetNonSpaceWrap(false)
		if db.textShow then
			dpTimeText:Show()
		else
			dpTimeText:Hide()
		end
		
		self:CheckVisibility(true)
		
		-- If we're not already running a timer, set some sane default values
		-- These are used when the user unlocks the bar, they can get an idea of
		-- what it'll look like. But we don't want to change these values when the
		-- bar's running!
		if not self.running and not self.locked then
			resetValues = true

			dpBar:SetValue(0.30)
			dpTimeText:SetText(L["Deadly"])
		end
	end
end

-- Checks whether or not to show our parent frame based on various conditions
-- Args: perform - true to call Hide or Show on the frame based on results
-- returns true if the frame should be shown
function mod:CheckVisibility(perform)
	local visible = false
	
	-- If we're running, we're visible. Simple as that.
	if self.running then
		visible = true
	else
		-- We're unlocked and want to give the user something to drag/test settings with
		if not self.locked then
			visible = true
		end
	end
	
	if perform then
		if visible then
			dpParent:Show()
		else
			dpParent:Hide()
		end
	end
	
	return visible
end

function mod:StartBar(length)
	self.startTime = GetTime()
	self.endTime = self.startTime + length

	dpBar:SetValue(length / 3)
	self.running = true	
	dpParent:Show()
end

-- ---------------------
-- Debugging
-- ---------------------

function mod:TestBar()
	if not self:IsEnabled() then return end
	
	dpBar:SetValue(1)
	self:StartBar(3)
end

-- ---------------------
-- Events
-- ---------------------

function mod:COMBAT_LOG_EVENT_UNFILTERED(_, _, eventType, srcGUID, srcName, _, dstGUID, dstName, _, spellID, spellName, ...)
	if srcGUID ~= UnitGUID("player") or spellName ~= spellInfo then return end
	if (eventType == "SPELL_PERIODIC_DAMAGE" or eventType == "SPELL_AURA_APPLIED")then
		tracking[dstGUID] = GetTime() --add/update mob in tracking table		
		if dstGUID == UnitGUID("target") then
			self:StartBar(3)
		end
	elseif eventType == "SPELL_AURA_REMOVED" then
		tracking[dstGUID] = nil
		if dstGUID == UnitGUID("target") then
			self.running = false
		end
	end
end

function mod:PLAYER_TARGET_CHANGED()
	self.running = false
	guid = UnitGUID("target")
	if tracking[guid] then
		length = (3 - (GetTime() - tracking[guid]) % 3.0) --find time remaining on current tick
		self:StartBar(length)
	end
end

do
	local function GetLSMIndex(t, value)
		for k, v in pairs(Media:List(t)) do
			if v == value then
				return k
			end
		end
		return nil
	end

	local function set(t, value)
		db.profile.deadly[t[#t]] = value
		self:ApplySettings()
	end
	local function get(t)
		return db.profile.deadly[t[#t]]
	end
	
	local function setcolor(t, ...)
		db.profile.deadly[t[#t]] = {...}
		self:ApplySettings()
	end
	local function getcolor(t)
		return unpack(db.profile.deadly[t[#t]])
	end
	
	local function dragstart()
		dpParent:StartMoving()
	end
	local function dragstop()
		db.profile.deadly.x = dpParent:GetLeft()
		db.profile.deadly.y = dpParent:GetBottom()
		dpParent:StopMovingOrSizing()
	end
	
	local function testbar()
		self:TestBar()
	end
	
	-- Select tables
	local positions = { L["Left"], L["Center"], L["Right"] }
	
	Envenomous.options.args.Deadly = {
		type = "group",
		name = L["Deadly"],
		desc = L["Deadly_Desc"],
		cmdHidden = true,
		disabled = function() return not self:IsEnabled() end,
		args = {
			desc = {
				type = "description",
				name = "  " .. L["Deadly_Desc"] .. "\n\n",
				order = 1,
				cmdHidden = true,
				image = "Interface\\Icons\\Ability_Rogue_DualWeild",
				imageWidth = 32, imageHeight = 32,
			},
			test = {
				type = "execute",
				name = "Test - DEBUGGING",
				desc = "Test the bar without actually having Deadly Poison up.",
				func = testbar,
				order = 4,
				hidden = true,
			},
			
			frame = {
				type = "group",
				name = L["Frame"],
				desc = nil,
				order = 100,
				inline = true,
				args = {
					lock = {
						type = "toggle",
						name = L["Lock"],
						desc = L["Toggle bar lock"],
						get = function(info)
							return self.locked
						end,
						set = function(info, v)
							self.locked = v
							if not self:IsEnabled() then return end
							if v then
								dpParent:EnableMouse(false)
								dpParent:SetScript("OnDragStart", nil)
								dpParent:SetScript("OnDragStop", nil)
							else
								dpParent:Show()
								dpParent:EnableMouse(true)
								dpParent:SetScript("OnDragStart", dragstart)
								dpParent:SetScript("OnDragStop", dragstop)
								self:ApplySettings()
							end
						end,
						order = 101,
					},
					blank1 = {
						type = "description",
						name = "",
						order = 102,
						cmdHidden = true,
						width = "full",
					},
					width = {
						type = "range",
						name = L["Width"],
						get = get, set = set,
						min = 10, max = 600, step = 1, bigStep = 5,
						order = 103,
					},
					height = {
						type = "range",
						name = L["Height"],
						get = get, set = set,
						min = 2, max = 50, step = 1,
						order = 104,
					},
					blank2 = {
						type = "description",
						name = "",
						order = 105,
						cmdHidden = true,
						width = "full",
					},
					scale = {
						type = "range",
						name = L["Scale"],
						get = get, set = set,
						min = 1, max = 150, step = 1, bigStep = 1,
						order = 106,
					},
					alpha = {
						type = "range",
						name = L["Alpha"],
						get = get, set = set,
						min = 0, max = 1, step = 0.1,
						order = 107,
						--width = "full",
					},
					blank3 = {
						type = "description",
						name = "",
						order = 108,
						cmdHidden = true,
						width = "full",
					},
					posX = {
						type = "input",
						name = L["X Position"],
						get = function(info) return tostring(db.profile.deadly.x) end,
						set = function(info, v)
							db.profile.deadly.x = tonumber(v)
							self:ApplySettings()
						end,
						order = 109,
					},
					posY = {
						type = "input",
						name = L["Y Position"],
						get = function(info) return tostring(db.profile.deadly.y) end,
						set = function(info, v)
							db.profile.deadly.y = tonumber(v)
							self:ApplySettings()
						end,
						order = 110,
					},
					blank4 = {
						type = "description",
						name = "",
						order = 111,
						cmdHidden = true,
						width = "full",
					},
					backColor = {
						type = "color",
						name = L["Background color"],
						get = getcolor, set = setcolor,
						hasAlpha = true,
						order = 112,
						--width = "full",
					},
					border = {
						type = "select",
						name = L["Border"],
						get = function(info) return GetLSMIndex("border", db.profile.deadly.border) end,
						set = function(info, v)
							db.profile.deadly.border = Media:List("border")[v]
							self:ApplySettings()
						end,
						values = Media:List("border"),
						order = 113,
						--width = nil,
					},
					blank5 = {
						type = "description",
						name = "",
						order = 114,
						cmdHidden = true,
						width = "full",
					},
					icon = {
						type = "toggle",
						name = L["Show icon"],
						get = function(info) return db.profile.deadly.icon end,
						set = function(info, v)
							db.profile.deadly.icon = v
							self:ApplySettings()
						end,
						order = 115,
					},
					iconPosition = {
						type = "select",
						name = L["Icon position"],
						get = get,
						set = function(info, v)
							db.profile.deadly.iconPosition = v
							self:ApplySettings()
						end,
						values = { positions[1], positions[3] },
						disabled = function() return not db.profile.deadly.icon end,
						order = 116,
					},
				}
			},
			
			bars = {
				type = "group",
				name = L["Bars"],
				order = 200,
				inline = true,
				args = {
					barColor = {
						type = "color",
						name = L["Bar color"],
						desc = L["Color of the bar"],
						get = getcolor, set = setcolor,
						hasAlpha = true,
						order = 201,
					},
					texture = {
						type = "select",
						name = L["Texture"],
						get = function(info) return GetLSMIndex("statusbar", db.profile.deadly.texture) end,
						set = function(info, v)
							db.profile.deadly.texture = Media:List("statusbar")[v]
							self:ApplySettings()
						end,
						values = Media:List("statusbar"),
						order = 205,
					},
				}
			},
			
			text = {
				type = "group",
				name = L["Text"],
				order = 300,
				inline = true,
				args = {
					textShow = {
						type = "toggle",
						name = L["Show text"],
						desc = L["Show countdown text."],
						get = get, set = set,
						order = 301,
					},
					blank1 = {
						type = "description",
						name = "",
						order = 302,
						cmdHidden = true,
						width = "full",
					},
					textColor = {
						type = "color",
						name = L["Text color"],
						desc = L["Text color"],
						get = getcolor,	set = setcolor,
						hasAlpha = true,
						order = 303,
					},
					textPosition = {
						type = "select",
						name = L["Text position"],
						desc = L["Text position"],
						get = get,
						set = function(info, v)
							db.profile.deadly.textPosition = v
							self:ApplySettings()
						end,
						values = positions,
						order = 304,
					},
					blank3 = {
						type = "description",
						name = "",
						order = 305,
						cmdHidden = true,
						width = "full",
					},
					textSize = {
						type = "range",
						name = L["Text size"],
						desc = L["Text size"],
						get = get, set = set,
						min = 6, max = 20, step = 1,
						order = 306,
					},
					textFont = {
						type = "select",
						name = L["Text font"],
						desc = L["Text font"],
						get = function(info) return GetLSMIndex("font", db.profile.deadly.textFont) end,
						set = function(info, v)
							db.profile.deadly.textFont = Media:List("font")[v]
							self:ApplySettings()
						end,
						values = Media:List("font"),
						order = 307,
					},
				}
			},
		},
	}
end