if (select(2, UnitClass("player"))) ~= "ROGUE" then return end

local mod = Envenomous:NewModule("Envenom", nil, "AceEvent-3.0", "AceConsole-3.0")
local Media = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Envenomous")
local self = mod
local db

local defaults = {
	profile = {
		envenom = {
			y = 350,
			
			width   = 250,
			height  = 16,
			alpha   = 1,
			scale   = 100.0,
			texture = "BantoBar",
			icon = true,
			iconPosition = 1,
			
			border    = "RothSquare",
			backColor = { 0, 0, 0, 1 },
			barColor = { 73/255, 109/255, 255/255, 0.8 },
			
			textShow     = true,
			textPosition = 2,
			textColor    = { 1, 1, 1, 1 },
			textFont     = "Emblem",
			textSize     = 14,
			
		}
	}
}

-- Frames
local locked = true
local envBar, envTimeText, envParent

-- Localized functions
local GetTime = _G.GetTime
local playerName = nil

-- Settings/infos
local resetValues = true
local spellInfo = GetSpellInfo(57993) -- Envenom R4

local function OnUpdate()
	if self.running then
		local currentTime = GetTime()
		local startTime = self.startTime
		local endTime = self.endTime
	
		local remainingTime = endTime - currentTime
		remainingTime = ((remainingTime > 0) and remainingTime or 0) -- Hackity hack hack
	
		if remainingTime == 0 then
			self.running = false
		else
			local perc = remainingTime / 6
			envBar:SetValue(perc)
			envTimeText:SetText(("%.1f"):format(remainingTime))
		end
	else
		if envParent:IsVisible() then
			self:CheckVisibility(true)
		end
	end
end
mod.OnUpdate = OnUpdate
local function OnShow()
	envParent:SetScript("OnUpdate", OnUpdate)
end
local function OnHide()
	envParent:SetScript("OnUpdate", nil)
end

function mod:OnInitialize()
	db = LibStub("AceDB-3.0"):New("EnvenomousDB", nil, "Default")
	self.db = db
	
	self.db:RegisterDefaults(defaults)
	
	self:SetEnabledState(false)
end

function mod:OnEnable()
	self:RegisterEvent("UNIT_AURA")
	
	playerName = UnitName("player")
	
	self.locked = locked
	self.startTime = 0
	self.endTime = 0
	self.running = false
	
	if not envParent then
		envParent = CreateFrame("Frame", nil, UIParent)
		envParent:SetFrameStrata("LOW")
		envParent:SetScript("OnShow", OnShow)
		envParent:SetScript("OnHide", OnHide)
		envParent:SetMovable(true)
		envParent:RegisterForDrag("LeftButton")
		envParent:SetClampedToScreen(true)
		
		envBar = CreateFrame("StatusBar", nil, envParent)
		envBar:SetFrameStrata("MEDIUM")
		envTimeText = envBar:CreateFontString(nil, "OVERLAY")
		
		envParent:Hide()
	end
	self:ApplySettings()
end

function mod:OnDisable()
	self.running = false
	self.locked = true
	envParent:Hide()
	envParent = nil
	
	self:UnregisterAllEvents()
end

-- ---------------------
-- Frame methods
-- ---------------------

function mod:ApplySettings()
	if not self:IsEnabled() then return end
	
	if envParent then
		local db = db.profile.envenom
		local back = {}
		
		-- envParent, to which all our stuff is anchored
		envParent:ClearAllPoints()
		if not db.x then
			db.x = (UIParent:GetWidth() / 2 - (db.width * (db.scale/100.0)) / 2) / (db.scale/100.0)
		end
		envParent:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", db.x, db.y)
		
		if db.border == "None" then
			envParent:SetWidth(db.width)
			envParent:SetHeight(db.height)
			
			back.bgFile = "Interface\\Tooltips\\UI-Tooltip-Background"
			back.tile = true
			back.tileSize = 16
			back.insets = { top = 0, right = 0, bottom = 0, left = 0 }
		else
			envParent:SetWidth(db.width + 9)
			envParent:SetHeight(db.height + 10)
			back.bgFile = "Interface\\Tooltips\\UI-Tooltip-Background"
			back.tile = true
			back.tileSize = 16
			back.edgeFile = Media:Fetch("border", db.border)
			back.edgeSize = 16
			back.insets = { top = 4, right = 4, bottom = 4, left = 4 }
		end
		envParent:SetBackdrop(back)
		envParent:SetBackdropColor(unpack(db.backColor))
		
		envParent:SetAlpha(db.alpha)
		envParent:SetScale(db.scale / 100.0)
		
		-- make us an icon!
		envParent.icon = envParent.icon or envParent:CreateTexture(nil, "OVERLAY")
		envParent.icon:ClearAllPoints()
		if db.iconPosition == 1 then
			envParent.icon:SetPoint("RIGHT", envParent, "LEFT", 0, 0)
		elseif db.iconPosition == 2 then
			envParent.icon:SetPoint("LEFT", envParent, "RIGHT", 0, 0)
		end
		envParent.icon:SetHeight(db.height + 5)
		envParent.icon:SetWidth(db.height + 5)
		envParent.icon:SetTexture([[Interface\ICONS\Ability_Rogue_Disembowel]])
		if db.icon then 
			envParent.icon:Show() 
		else
			envParent.icon:Hide()
		end
		
		-- envBar, the actual timer bar
		envBar:ClearAllPoints()
		envBar:SetPoint("CENTER", envParent, "CENTER")
		envBar:SetWidth(db.width)
		envBar:SetHeight(db.height)
		envBar:SetStatusBarTexture(Media:Fetch("statusbar", db.texture))
		envBar:SetMinMaxValues(0, 1)
		envBar:SetStatusBarColor(unpack(db.barColor))
		envBar:Show()
		
		-- envTimeText, countdown timer text
		envTimeText:ClearAllPoints()
		if db.textPosition == 1 then -- LEFT
			envTimeText:SetPoint("LEFT", envParent, "LEFT", 5, 0)
			envTimeText:SetJustifyH("LEFT")
		elseif db.textPosition == 2 then -- CENTER
			envTimeText:SetPoint("CENTER", envParent, "CENTER")
			envTimeText:SetJustifyH("CENTER")
		elseif db.textPosition == 3 then -- RIGHT
			envTimeText:SetPoint("RIGHT", envParent, "RIGHT", -5, 0)
			envTimeText:SetJustifyH("RIGHT")
		end
		envTimeText:SetFont(Media:Fetch("font", db.textFont), db.textSize)
		envTimeText:SetTextColor(unpack(db.textColor))
		envTimeText:SetShadowColor(0, 0, 0, 1)
		envTimeText:SetShadowOffset(0.8, -0.8)
		envTimeText:SetNonSpaceWrap(false)
		if db.textShow then
			envTimeText:Show()
		else
			envTimeText:Hide()
		end
		
		self:CheckVisibility(true)
		
		-- If we're not already running a timer, set some sane default values
		-- These are used when the user unlocks the bar, they can get an idea of
		-- what it'll look like. But we don't want to change these values when the
		-- bar's running!
		if not self.running and not self.locked then
			resetValues = true

			envBar:SetValue(0.30)
			envTimeText:SetText(L["Envenom"])
		end
	end
end

-- Checks whether or not to show our parent frame based on various conditions
-- Args: perform - true to call Hide or Show on the frame based on results
-- returns true if the frame should be shown
function mod:CheckVisibility(perform)
	local visible = false
	
	-- If we're running, we're visible. Simple as that.
	if self.running then
		visible = true
	else
		-- We're unlocked and want to give the user something to drag/test settings with
		if not self.locked then
			visible = true
		end
	end
	
	if perform then
		if visible then
			envParent:Show()
		else
			envParent:Hide()
		end
	end
	
	return visible
end

function mod:StartBar(length)
	self.startTime = GetTime()
	self.endTime = self.startTime + length
	self.running = true
	
	envBar:SetValue(length / 6)
	envParent:Show() -- Might not be shown if potentialShow is disabled
end

-- ---------------------
-- Debugging
-- ---------------------

function mod:TestBar()
	if not self:IsEnabled() then return end

	self:StartBar(6)
end

-- ---------------------
-- Events
-- ---------------------

function mod:UNIT_AURA(event, unit)
	if unit ~= "player" then return end
	local name, _, _, _, _, duration, endTime = UnitBuff(unit, spellInfo)
	if name then
		self.startTime = endTime - duration
		self.endTime = endTime
		self.running = true
		envBar:SetValue((endTime - GetTime()) / 6)
		envParent:Show()
	end
end

do
	local function GetLSMIndex(t, value)
		for k, v in pairs(Media:List(t)) do
			if v == value then
				return k
			end
		end
		return nil
	end

	local function set(t, value)
		db.profile.envenom[t[#t]] = value
		self:ApplySettings()
	end
	local function get(t)
		return db.profile.envenom[t[#t]]
	end
	
	local function setcolor(t, ...)
		db.profile.envenom[t[#t]] = {...}
		self:ApplySettings()
	end
	local function getcolor(t)
		return unpack(db.profile.envenom[t[#t]])
	end
	
	local function dragstart()
		envParent:StartMoving()
	end
	local function dragstop()
		db.profile.envenom.x = envParent:GetLeft()
		db.profile.envenom.y = envParent:GetBottom()
		envParent:StopMovingOrSizing()
	end
	
	local function testbar()
		self:TestBar()
	end
	
	-- Select tables
	local positions = { L["Left"], L["Center"], L["Right"] }
	
	Envenomous.options.args.envenom = {
		type = "group",
		name = L["Envenom"],
		desc = L["Envenom_Desc"],
		cmdHidden = true,
		disabled = function() return not self:IsEnabled() end,
		args = {
			desc = {
				type = "description",
				name = "  " .. L["Envenom_Desc"] .. "\n\n",
				order = 1,
				cmdHidden = true,
				image = "Interface\\Icons\\Ability_Rogue_Disembowel",
				imageWidth = 32, imageHeight = 32,
			},
			test = {
				type = "execute",
				name = "Test - DEBUGGING",
				desc = "Test the bar without actually having Envenom up.",
				func = testbar,
				order = 4,
				hidden = true,
			},
			
			frame = {
				type = "group",
				name = L["Frame"],
				desc = nil,
				order = 100,
				inline = true,
				args = {
					lock = {
						type = "toggle",
						name = L["Lock"],
						desc = L["Toggle bar lock"],
						get = function(info)
							return self.locked
						end,
						set = function(info, v)
							self.locked = v
							if not self:IsEnabled() then return end
							if v then
								envParent:EnableMouse(false)
								envParent:SetScript("OnDragStart", nil)
								envParent:SetScript("OnDragStop", nil)
							else
								envParent:Show()
								envParent:EnableMouse(true)
								envParent:SetScript("OnDragStart", dragstart)
								envParent:SetScript("OnDragStop", dragstop)
								self:ApplySettings()
							end
						end,
						order = 101,
					},
					blank1 = {
						type = "description",
						name = "",
						order = 102,
						cmdHidden = true,
						width = "full",
					},
					width = {
						type = "range",
						name = L["Width"],
						get = get, set = set,
						min = 10, max = 600, step = 1, bigStep = 5,
						order = 103,
					},
					height = {
						type = "range",
						name = L["Height"],
						get = get, set = set,
						min = 2, max = 50, step = 1,
						order = 104,
					},
					blank2 = {
						type = "description",
						name = "",
						order = 105,
						cmdHidden = true,
						width = "full",
					},
					scale = {
						type = "range",
						name = L["Scale"],
						get = get, set = set,
						min = 1, max = 150, step = 1, bigStep = 1,
						order = 106,
					},
					alpha = {
						type = "range",
						name = L["Alpha"],
						get = get, set = set,
						min = 0, max = 1, step = 0.1,
						order = 107,
						--width = "full",
					},
					blank3 = {
						type = "description",
						name = "",
						order = 108,
						cmdHidden = true,
						width = "full",
					},
					posX = {
						type = "input",
						name = L["X Position"],
						get = function(info) return tostring(db.profile.envenom.x) end,
						set = function(info, v)
							db.profile.envenom.x = tonumber(v)
							self:ApplySettings()
						end,
						order = 109,
					},
					posY = {
						type = "input",
						name = L["Y Position"],
						get = function(info) return tostring(db.profile.envenom.y) end,
						set = function(info, v)
							db.profile.envenom.y = tonumber(v)
							self:ApplySettings()
						end,
						order = 110,
					},
					blank4 = {
						type = "description",
						name = "",
						order = 111,
						cmdHidden = true,
						width = "full",
					},
					backColor = {
						type = "color",
						name = L["Background color"],
						get = getcolor, set = setcolor,
						hasAlpha = true,
						order = 112,
						--width = "full",
					},
					border = {
						type = "select",
						name = L["Border"],
						get = function(info) return GetLSMIndex("border", db.profile.envenom.border) end,
						set = function(info, v)
							db.profile.envenom.border = Media:List("border")[v]
							self:ApplySettings()
						end,
						values = Media:List("border"),
						order = 113,
						--width = nil,
					},
					blank5 = {
						type = "description",
						name = "",
						order = 114,
						cmdHidden = true,
						width = "full",
					},
					icon = {
						type = "toggle",
						name = L["Show icon"],
						get = function(info) return db.profile.envenom.icon end,
						set = function(info, v)
							db.profile.envenom.icon = v
							self:ApplySettings()
						end,
						order = 115,
					},
					iconPosition = {
						type = "select",
						name = L["Icon position"],
						get = get,
						set = function(info, v)
							db.profile.envenom.iconPosition = v
							self:ApplySettings()
						end,
						values = { positions[1], positions[3] },
						disabled = function() return not db.profile.envenom.icon end,
						order = 116,
					},
				}
			},
			
			bars = {
				type = "group",
				name = L["Bars"],
				order = 200,
				inline = true,
				args = {
					barColor = {
						type = "color",
						name = L["Bar color"],
						desc = L["Color of the bar"],
						get = getcolor, set = setcolor,
						hasAlpha = true,
						order = 201,
					},
					texture = {
						type = "select",
						name = L["Texture"],
						get = function(info) return GetLSMIndex("statusbar", db.profile.envenom.texture) end,
						set = function(info, v)
							db.profile.envenom.texture = Media:List("statusbar")[v]
							self:ApplySettings()
						end,
						values = Media:List("statusbar"),
						order = 205,
					},
				}
			},
			
			text = {
				type = "group",
				name = L["Text"],
				order = 300,
				inline = true,
				args = {
					textShow = {
						type = "toggle",
						name = L["Show text"],
						desc = L["Show countdown text."],
						get = get, set = set,
						order = 301,
					},
					blank1 = {
						type = "description",
						name = "",
						order = 302,
						cmdHidden = true,
						width = "full",
					},
					textColor = {
						type = "color",
						name = L["Text color"],
						desc = L["Text color"],
						get = getcolor,	set = setcolor,
						hasAlpha = true,
						order = 303,
					},
					textPosition = {
						type = "select",
						name = L["Text position"],
						desc = L["Text position"],
						get = get,
						set = function(info, v)
							db.profile.envenom.textPosition = v
							self:ApplySettings()
						end,
						values = positions,
						order = 304,
					},
					blank3 = {
						type = "description",
						name = "",
						order = 305,
						cmdHidden = true,
						width = "full",
					},
					textSize = {
						type = "range",
						name = L["Text size"],
						desc = L["Text size"],
						get = get, set = set,
						min = 6, max = 20, step = 1,
						order = 306,
					},
					textFont = {
						type = "select",
						name = L["Text font"],
						desc = L["Text font"],
						get = function(info) return GetLSMIndex("font", db.profile.envenom.textFont) end,
						set = function(info, v)
							db.profile.envenom.textFont = Media:List("font")[v]
							self:ApplySettings()
						end,
						values = Media:List("font"),
						order = 307,
					},
				}
			},
		},
	}
end